#!/bin/sh
# ============================================================================
#
# This file is part of the 'FFP-stick for ZyXEL NSA-nnn'
# http://zyxel.nas-central.org/wiki/FFP-stick
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#
# This script has multiple functions.
# - Download, unpack and start ffp
# - Stop ffp
# - Execute any function outside the ffp chroot
#
# It is supposed to be started by usb_key_func.sh/fun_plug.sh or by itself.
#
# Execute the script without params for a brief help

# ===========================================
# Options 
# ===========================================
# Uncomment next line if you want a /ffp symlink
# FFPSYMLINK=1
#
# Uncomment next line if you don't want to chroot (you'll need the /ffp symlink too).
# NOCHROOT=1
# (The script will chroot anyway if the root is read only, or if you 
# didn't enable the /ffp symlink
#
# Uncomment next line if you want the rootpassword changed, when not chrooting
# CHANGEROOTPASSWORD=1
# Edit the function ChangePasswords() to add your users/passwords/hashes
# 
# Choose which ffp version to use. 0.7/arm, 0.7/oarm, 0.5, 0.6 or auto. 
# Currently 'auto' means 0.7/arm for all boxes which support EABI, and 0.7/oarm for all other boxes
FFPVERSION=auto
#
# ===========================================
# End of options
# ===========================================

SCRIPTVERSION="20120306"

# To be filled by commandline options:
FFPROOT=
NASTYPE=
FIRMWARE=
CHROOTDIR=
#

HELPERDIR=/tmp/.ffpstick/
HELPERFILE=helper.inc

# ===================================================
# function SplitPath
# split path in first parameter in path and filename
SPLITTEDPATH="" # Output path
SPLITTEDFILE="" # Output file
# environment: current directory changes
# ===================================================
SplitPath()
{
        SPLITTEDPATH=""
        SPLITTEDFILE=""
        IFSBACKUP=${IFS};   IFS=/                       
                   
        for chunk in $1; do
    		newpath="" 
                if [ "${SPLITTEDPATH}" = "" ] ; then
            		newpath="${chunk}"
                else        
            		newpath="${SPLITTEDPATH}/${chunk}"
                fi          
    		if [ "${newpath}" = "$1" ] ; then
            		SPLITTEDFILE="${chunk}"
                elif [ "/${newpath}" = "$1" ] ; then
            		SPLITTEDPATH="/${SPLITTEDPATH}"
            		SPLITTEDFILE="${chunk}"
                else        
            		SPLITTEDPATH="${newpath}"
        	fi          
        done                                            
	IFS=${IFSBACKUP}    
	if [ "" != "${SPLITTEDPATH}" ]; then
	    cd ${SPLITTEDPATH}
	fi                  
	SPLITTEDPATH=` pwd `
}                           

# =============================================================
# function CleanUp 
# Delete some unneeded (and possibly dangerous) DNS-323 scripts
# =============================================================
CleanUp()
{
    echo -ne Delete some DNS-323 specific stuff...
    for file in start/passwd.sh start/rcS.sh start/shells.sh sbin/store-passwd.sh
    do
	rm -f ffp/${file}
    done
    echo done
}

# =============================================================
# function IsAbiSupported
# Checks if oabi or eabi is supported
# Uses the external executable testeabi or testoabi
# 1st argument: test or bg (internal use)
# 2nd argument eabi or oabi
# =============================================================
IsAbiSupported()
{
    if [ "$1" = "bg" ]; then
	echo no >/tmp/$2.status
	if ${FFPROOT}/bin/test$2 2>/dev/null ; then
	    echo yes >/tmp/$2.status
	fi
	return
    fi

    IsAbiSupported bg $2 &
    sleep 5
    killall test$2 >/dev/null 2>&1
    supported=` cat /tmp/$2.status `
    rm /tmp/$2.status
    if [ "${supported}" = "no" ] ; then
	return 1
    fi
    return 0
}

# =============================================================
# function Unpack_fun_plug
# Unpack fonz' base package
# 1st argument: filename
# =============================================================
Unpack_fun_plug()
{
    echo Unpack ${FFPROOT}/$1...
    cd ffp
    tar -xzf ../$1
    cd ..
    rm $1
    echo Done
    CleanUp
}

# =============================================================
# function CheckForFfpTarball
# Check for a user-provided ffp base package either in /path/to/ffproot
# or it's parent (the ext3 partition of the ffp-stick)
# =============================================================
CheckForFfpTarball()
{
    # find ffp tarball
    for file in fun_plug.tgz ../fun_plug.tgz
    do
	if [ -r ${file} ]; then
	    Unpack_fun_plug ${file}
	fi
    done
}

# =============================================================
# function TryToDownload
# Downloads ffp base package from inreto.de
# =============================================================
TryToDownload()
{
#############################################
# Option for ffp 0.6 added at 2012-01-10
# as version 0.5 denied to run on the
# Medion P89626/P89630
# Option for ffp 0.7 added at 2012-01-31
# Option for ffp 0.7/oabi added at 2012-03-06
#############################################
    FFP05=http://ffp.inreto.de/ffp/0.5/
    FFP06=http://ffp.inreto.de/testing/ffp/0.6/arm/
    FFP07arm=http://ffp.inreto.de/ffp/0.7/arm/
    FFP07oarm=http://ffp.inreto.de/ffp/0.7/oabi/

    # Check if a stock wget is available:
    WGET=` which wget `
    
    if [ "" = "${WGET}" ] ; then
	# if not, use the one from the stick
	WGET=${FFPROOT}/bin/wget
    fi

    if ! [ -x ${WGET} ]; then
	echo No wget found
	return 0
    fi

    FFPURL=${FFP05}
    
    case ${FFPVERSION} 
    in
	0.5*)	FFPURL=${FFP05}
		;;
	0.6*)	FFPURL=${FFP06}
		;;
	0.7/arm*)
		FFPURL=${FFP07arm}
		;;
	0.7/oarm*)
		FFPURL=${FFP07oarm}
		;;
	*)	echo Which FFP version to use?	
		echo -ne Is OABI supported?...
		if IsAbiSupported test oabi ; then
		    echo -ne "yes. "
		else
		    echo -ne "no. "
		fi
		echo -ne Is EABI supported?...
		if IsAbiSupported test eabi ; then
		    echo yes, choosing FFP 0.7/arm
		    FFPURL=${FFP07arm}
		    FFPVERSION=0.7/arm
		else
		    echo no, choosing FFP 0.7/oarm
		    FFPURL=${FFP07oarm}
		    FFPVERSION=0.7/oarm
		fi
		;;
    esac

    FFPURL=${FFPURL}fun_plug.tgz
    echo Will try to download fun_plug.tgz \(version ${FFPVERSION}\)
    echo ${FFPURL}
    echo If the script stops here, downloading the tarball from ffp.inreto.de failed.
    echo You can try to get internet access for the ${NASTYPE},
    echo "or copy fun_plug.tgz (case sensitive!) to the share containing ffproot."
    echo It should be recognized and installed.
    echo It is possible that you have to reboot the box before this share shows up.
    echo ping ffp.inreto.de...
    while ! [ -r ffp/bin/busybox ] && ! ping -c 1 ffp.inreto.de >/dev/null ; do
	sleep 10
	CheckForFfpTarball
    done  

    if ! [ -r ffp/bin/busybox ]; then
	echo www.inreto.de found. Downloading fun_plug.tgz...
	cd ffp
	${WGET} -qO- ${FFPURL} | tar -xz
	if [ "$?" != "0" ] ; then
    	    echo Download failed.
	else
	    echo fun_plug.tgz is downloaded
	    CleanUp
	fi
    	cd ..
    fi
    
    SITESFILE=ffp/etc/funpkg/sites
    if [ -r ${SITESFILE} ] ; then
	FFPVERSION2=${FFPVERSION}
	if [ "${FFPVERSION2}" = "0.7/oarm" ]; then
	    FFPVERSION2=0.7/oabi
	fi
    
	echo "z	http://downloads.zyxel.nas-central.org/Users/Mijzelf/FFP-Stick/packages/${FFPVERSION2}" >> ${SITESFILE}
    fi
    
    # Put the ffp version in the etc directory:
    echo FFPVERSION=${FFPVERSION} >ffp/etc/ffp.version
    
    # Say hello to the author of this script. This doesn't do anything, 
    # (the file doesn't exist, so you'll get a 404), except generating 
    # a logline on my webserver. (When it's online), Then I know that yet 
    # another FFP-stick is created. I like that. ;)
    # If you want to opt-out, just remove the line below
    ${WGET} -qO- http://mijzelf.hopto.org/hello_author.php?nas=${NASTYPE}\&firmware=${FIRMWARE}\&ffp=${FFPVERSION} >/dev/null 2>&1
    if [ -r ./bin/wget ] ; then
        rm ./bin/wget
    fi
}

#####################
# Function added 20101217 to solve the problem that the /i-data/md0
# symlink was not copied right on the NSA-320
#####################
CopySymlink()
{
    echo CopySymlink $1 $2
    target=` ls -l $1 | awk -F\  '{print $NF}' `    
    echo ln -s $target $2
    ln -s $target $2
}

# ===========================================================
# function CopyTree
# Helper function for the NSA3x0. 
# ===========================================================
CopyTree() 
{
    echo CopyTree $1 $2 $3
    
    if [ -h $2 ] ; then
	# Directory turns out to be a symlink. Just copy it
	CopySymlink $2 $3
	return 0
    fi
    
    mkdir $3
    if mountpoint $2 ; then
	# Directory is a mountpoint
	mount --bind $2 $3
	if [ "0" != "$?" ] ; then
	    return 1
	fi
	return 0
    fi 
	
    for file in ` ls $2 `
    do
	if [ -d $2/${file} ] ; then
	# File is a directory
	    CopyTree $1 $2/${file} $3/${file}
	    if [ "0" != "$?" ] ; then
	        return 1
	    fi
	#########################################
	# Added 2011-03-08; Changed 2011-03-23;
	# The NSA-320 has a symlink /etc/exports, 
	# which points to a non-existing file
	# as long as the nfs packet is not installed.
	# This frustrated this copy script
	elif [ -h $2/${file} ] ; then
	    # File is a symlink. Just copy it
	    CopySymlink $2/${file} $3/${file}
	#########################################
	else
	    case $1 in
		ln)
		    ln $2/${file} $3/${file}
		    ;;
		cp)
		    cp -p $2/${file} $3/${file}
		    ;;
	    esac
	    if [ "0" != "$?" ] ; then
		return 1
	    fi 
	fi
    done
    return 0
}
	     
# ===================================================================
# function NSA320BindAll
# Helper function for the NSA320. Because the firmware directories cannot
# be bind-mounted on the FFP-stick directories, we'll inverse it. 
# Basically the rootfs is doubled using hardlinks, and the 2nd copy is 
# chrooted, with the FFP-stick bindmounted on /2ndcopy/ffp
# ===================================================================
NSA320BindAll()
{
    chdir=/tmp/bindroot
    
    mkdir ${chdir}
    for dir in bin sbin lib usr i-data e-data dev mnt
    do
	CopyTree ln /${dir} ${chdir}/${dir}
	if [ "0" != "$?" ]; then
	    echo Failed to CopyTree /${dir}
	    return 1
	fi
    done

    mkdir ${chdir}/tmp
    mount -t tmpfs tmfs ${chdir}/tmp
    if [ "0" != "$?" ]; then
	echo Failed to mount tmpfs on ${chdir}/tmp
        return 1
    fi

    echo ` date ` Waiting for fadd to show up...
    while ! ps | grep /sbin/fadd >/dev/null ; do
	sleep 10
    done
    echo ` date ` Hello fadd!

    CopyTree cp /etc ${chdir}/tmp/.etc
    if [ "0" != "$?" ]; then
	echo Failed to CopyTree /etc
        return 1
    fi

    # Because the files in /etc are bound to be changed by the firmware, we can't just make hardlinks to
    # it. We'll just hijack the map...
    mount --bind ${chdir}/tmp/.etc /etc
    if [ "0" != "$?" ]; then
	echo Failed to bind ${chdir}/tmp/.etc on /etc
        return 1
    fi
    
    ##################
    # Added 20101221 to solve the problem that the 'default' shares were not 
    # exported, only the user made ones. The mount mtd4 on /etc/zyxel was lost.
    echo mount --bind ${chdir}/tmp/.etc/zyxel /etc/zyxel
    mount --bind ${chdir}/tmp/.etc/zyxel /etc/zyxel
    ##################
    
    ln -s ${FFPROOT}/ffp ${chdir}/ffp
    ln -s ${FFPROOT}/ffp/etc ${chdir}/etc
    ln -s ${FFPROOT}/after_booting.sh ${chdir}/after_booting.sh
    
    mount --bind ${chdir}/tmp/.etc ${chdir}${FFPROOT}/ffp/etc/original
    mount --bind ${chdir}/tmp/.etc/zyxel ${chdir}${FFPROOT}/ffp/etc/zyxel

    # We'll hijack /var/log to, for the same reason...
    ln -s ${FFPROOT}/var ${chdir}/var
    CopyTree cp /var/log ${chdir}/tmp/.log
    mount --bind ${chdir}/tmp/.log /var/log
    mount --bind ${chdir}/tmp/.log ${chdir}${FFPROOT}/var/log

    if ! [ -d ${chdir}/dev/pts ]; then 
	mkdir ${chdir}/dev/pts
    fi

    mount -t devpts devpts ${chdir}/dev/pts
    mkdir ${chdir}/proc
    mount -t proc proc ${chdir}/proc
    mkdir ${chdir}/sys
    mount -t sysfs sys ${chdir}/sys

    mkdir ${chdir}/home
    mount --bind ${FFPROOT}/home ${chdir}/home

    # Restart Samba because it has gone mad about the etc exchange
    /etc/init.d/samba.sh restart

    CHROOTDIR=${chdir}
    return 0
}

# ========================================================
# function ClassicBindAll
# All firmware root directories are bind mounted on ffproot
# ========================================================
ClassicBindAll()
{
    # mount directories for chroot
    tomount="e-data i-data"
    for arg in rbind bind
    do
	for tm in ${tomount}
	do
	    mount --${arg} /${tm} ${tm}
	    if [ "$?" != "0" ] ; then
		return 1
	    fi
	done
	tomount="dev var tmp bin sbin lib usr"
    done

    mount --bind /etc ffp/etc/original
    mount --bind /etc/zyxel ffp/etc/zyxel
    mount -t proc none proc
    mount -t sysfs none sys
    CHROOTDIR=${FFPROOT}
    return 0
}

# ================================================
# function FinalBindAll
# 'poor man's chroot'
# FFP is chrooted with hardly any connection to the firmware
# ================================================
FinalBindAll()
{
    CopyTree cp /dev ${FFPROOT}/dev
    if ! [ -d ${FFPROOT}/dev/pts ] ; then
        mkdir ${FFPROOT}/dev/pts
    fi
    mount -t devpts devpts ${FFPROOT}/dev/pts
    for dir in i-data e-data
    do
	mkdir ${FFPROOT}/${dir}
	CopyTree cp /${dir} ${FFPROOT}/${dir} 
    done
    mount -t proc proc proc
    mount -t sysfs sys sys
    CHROOTDIR=${FFPROOT}
}

WriteParams()
{
    echo NASTYPE=${NASTYPE} >$1
    echo FIRMWARE=${FIRMWARE} >>$1
    echo FFPROOT=${FFPROOT} >>$1
    echo CHROOTDIR=${CHROOTDIR} >>$1
}

# ==========================================================
# function ChangePasswords()
# Changes the default password for root (the default is 'disabled')
# and adds users and password if you like
# ==========================================================
ChangePasswords()
{
    echo -ne Adding/changing passwords...
    
    # You can add your own users here:
    for name in user root
   
    do
        if ! cat /etc/passwd | grep ^${name} ; then
	    # User doesn't exist yet, so add it
	    adduser -G nobody -s /ffp/bin/sh ${name}
	fi
	# Delete password, so the shadow entry starts with name::
	passwd -d ${name}
	old=${name}::

	hash=""
	password=""
	# You can specify the password per user, or specify the hash value, 
	# which is safer. Don't forget to escape your $
	case ${name} in
	    user)
	        password=user
	        ;;
	    root)
	        # password=root
		hash=\$1\$\$oCLuEVgI1iAqOA8pwkzAg1
	        ;;
	    *)
		password=${name}
		;;
	esac
			
	if [ "" = "${hash}" ]; then
	    # If hash is empty, create it:
	    tempfile=/tmp/$$` date +%s `.tmp
	    makepwd ${password} | sed -e 's/\//\\\//g' >${tempfile}
	    hash=` cat ${tempfile} `
            rm ${tempfile}
        fi
	
	new=${name}:${hash}:
        mv /etc/shadow /tmp/shadow.bak
        sed 's|'${old}'|'${new}'|' </tmp/shadow.bak >/etc/shadow
        rm /tmp/shadow.bak
    done
    echo done
}

# ===========================================
# function IsFFPinstalled()
# Checks for existance of FFP, to decide to 
# download or not
# ===========================================
IsFFPinstalled()
{
    if [ -r ffp/bin/busybox ] ; then
	return 0
    fi
    if [ -r ffp/bin/busybox-netutils ] ; then
	return 0
    fi
    return 1
}

# ===========================================
# function StartFFP()
# main function. Will download ffp if necessary
# create an environment for chrooting,
# chroot ffp
# ===========================================
StartFFP()
{
    # create rootdirectory
    if [ -r rootfs.tgz ]; then
	echo -ne Extract rootfs.tgz...
        tar -xzf rootfs.tgz
	rm rootfs.tgz
	echo Done
    fi

    CheckForFfpTarball
    
    if ! IsFFPinstalled ; then
	TryToDownload
    fi

    if ! IsFFPinstalled ; then
	echo Could not download fun_plug.tgz for some reason
	echo Will start polling for the existance forever in the share containing ffproot...
	while ! IsFFPinstalled ; do
	    sleep 10
	    CheckForFfpTarball
	done
    fi

    if [ "${FFPSYMLINK:-notset}" != "notset" ] ; then
	echo -ne Try to create a /ffp symlink...
	ln -s ${FFPROOT}/ffp /ffp
	if [ -x /ffp/bin/busybox ]; then
	    echo done
	else
	    echo "Failed. (Read only rootfs?)"
	fi
    fi
   
    if [ "${NOCHROOT:-notset}" != "notset" ] ; then
	if [ -x /ffp/bin/busybox ] ; then
	    NOCHROOT=1	
	    if [ "${CHANGEROOTPASSWORD:-notset}" != "notset" ] ; then
		ChangePasswords
	    fi
	else
	    echo Cannot answer to the 'nochroot' request. There is no /ffp symlink
	    NOCHROOT=0
	fi
    else
	NOCHROOT=0
    fi

    if [ ${NOCHROOT} -eq 0 ] ; then
        ClassicBindAll
	if [ "$?" != "0" ]; then
    	    echo "Bummer! Cannot bind directories. Will try the NSA320 hack to get it going"
	    NSA320BindAll
    	    if [ "$?" != "0" ]; then
		echo "The NSA320 hack doesn't work either. Will do the best I can to get it working"
		echo Possibilities are limited.
		FinalBindAll
	    fi
	fi

        echo Will chroot ffp on ${CHROOTDIR}
    else
	CHROOTDIR=""
    fi
    
    # Generate helper file 
    hf=${CHROOTDIR}${HELPERDIR}
    echo -ne Generate helper file ${hf}... 
    mkdir -p ${hf}
    chmod 755 ${hf}
    WriteParams ${hf}${HELPERFILE}
    echo done

    # Extend /etc/profile
    if ! cat ${CHROOTDIR}/ffp/etc/profile | grep FFPVERSION
    then
cat >>${CHROOTDIR}/ffp/etc/profile <<_EOF_
. ${HELPERDIR}${HELPERFILE}
. /ffp/etc/ffp.version
echo "== Welcome at FFP version \${FFPVERSION}, running on \${NASTYPE} \` hostname \`, firmware \${FIRMWARE} =="
_EOF_
    fi

    CHROOTCOMMAND=""
    if [ "" != "${CHROOTDIR}" ] ; then
        # start helper
	echo Start execute_outside_chroot Helper...
	$0 InvokeHelper -c ${CHROOTDIR} &
	sleep 1
	echo done
	CHROOTCOMMAND="chroot ${CHROOTDIR}"
    fi

    # Copy /i-data/md0 symlink to /mnt/HD_a2 to make the ZyXEL more compatible with the DNS-323
    if [ -h /i-data/md0 ] ; then
	CopySymlink /i-data/md0 ${CHROOTDIR}/mnt/HD_a2
    fi

    # run fun_plug local, if present
    if [ -x ${CHROOTDIR}/ffp/etc/fun_plug.local ]; then
	${CHROOTCOMMAND} /ffp/etc/fun_plug.local
    fi
 
    # start services
    if [ -x ${CHROOTDIR}/ffp/etc/rc ]; then
        exec ${CHROOTCOMMAND} /ffp/etc/rc
    fi

    exit 0
}

# ============================================
# function StopFFP()
# terminates all ffp processes,
# unmounts all ffp mounts
# ============================================
StopFFP()
{
    if [ "${CHROOTDIR}" = "" ]; then
	if [ -x ${CHROOTDIR}/ffp/etc/rc ]; then
	    echo /ffp/etc/rc stop
	    /ffp/etc/rc stop
	fi
    else
        cd /
	# stop services
	if [ -x ${CHROOTDIR}/ffp/etc/rc ]; then
	    echo /ffp/etc/rc stop
	    chroot ${CHROOTDIR} /ffp/etc/rc stop
	fi
    fi
    sleep 1

    message="Try to kill all remaining chrooted processes"
    for killtype in SIGTERM SIGKILL
    do    
	# List of all programs pid
	tokill=` ps | awk '{print $1}' `
	for pf in $tokill
	do
	    if [ -f /proc/$pf/maps ]; then
		if grep ${CHROOTDIR}/ffp/lib /proc/$pf/maps >/dev/null ; then                    
		    # Program has some chrooted libraries loaded
		    if [ "${message}" != "" ] ; then
			echo ${message} ; message=""
		    fi
            	    echo "      " ` cat /proc/$pf/cmdline `
            	    kill -${killtype} $pf
        	fi  
    	    fi          
	done                
        sleep 1
	message="Try a bit harder"
    done

    if [ "${CHROOTDIR}" = "" ] ; then
	exit 0
    fi

    echo Unmount all FFP mounts...
    tounmount=` cat /proc/mounts | grep ${CHROOTDIR} | awk '{print $2}' 2>/dev/null `
    while [ "${tounmount}" != "" ]; do
	# While loop because mounts are nested.
	for unmount in ${tounmount} ; do
	    if umount $unmount 2>/dev/null ; then
	        echo "      " $unmount
	    fi  
	done        
	tounmount=` cat /proc/mounts | grep ${CHROOTDIR} | awk '{print $2}' 2>/dev/null `
    done
    echo done        
    exit 0
}

# ==================================================
# function InvokeHelper()
# Helperfunction to be able to run programs outside the chroot.
# It will sleep until it's woken up by a SIGCONT,
# then in will execute scripts in HELPERDIR
# ==================================================
HelperLog()
{
    echo ` date +"%b %d %H:%m:%S" `: Helper script: $*
}
InvokeHelper()
{
    hd=${CHROOTDIR}${HELPERDIR}
    hf=${hd}${HELPERFILE}

    mv ${hf} ${hf}.tmp
    cat ${hf}.tmp | grep -v HELPERPID >${hf}
    rm ${hf}.tmp

    if [ "$1" = "stop" ]; then
	exit 0
    fi

    echo HELPERPID=$$ >>${hf}

    HelperLog started with pid $$
    
    while [ true ]; do
        trap "HelperLog Stop" SIGTERM
	if [ ! -d ${hd} ] ; then
	    trap "HelperLog Terminated" SIGTERM
	    exit 0
	fi
	
	local scripts=` ls ${hd} | grep helper | grep .sh `
	if [ "${scripts}" != "" ] ; then
	    trap "HelperLog Terminated" SIGTERM

	    for script in ${scripts}
	    do
		HelperLog Executing ${script}
		tmpfile=/tmp/$$` date +%s `
		mv ${hd}${script} ${tmpfile}
		chmod a+x ${tmpfile}
		${tmpfile}
	    	rm ${tmpfile}
	    done
	else
	    HelperLog No more jobs to do\; Sleeping
	    kill -SIGSTOP $$
	fi
    done
}

PrintHelp()
{
    echo Usage: $0 command arguments
    echo "   command is one of:"
    echo "       StartFFP"
    echo "       StopFFP"
    echo "       InvokeHelper"
    echo "       Help"
    echo "   arguments:"
    echo "       -f firmware"
    echo "       -n nastype"
    echo "       -r ffproot"
    echo "       -c chrootdir"
}

ReadParams()
{
    shift
    while [ "$1" != "" ] ; do
	case $1 in
	    -f)
		FIRMWARE=$2
		shift
		;;
	    -n)
		NASTYPE=$2
		shift
		;;
	    -h)
		PrintHelp
		shift
		;;
	    -r)
		FFPROOT=$2
		shift
		;;
	    -c)
		CHROOTDIR=$2
		shift
		;;
	    *)
		echo Unknown option $1
		;;
	esac
	shift
    done

    if [ "" = "${CHROOTDIR}" ]; then
	CHROOTDIR=${FFPROOT}
    fi

    hf=${CHROOTDIR}${HELPERDIR}${HELPERFILE}
    if [ -r ${hf}  ]; then
        . ${hf}
    fi
}

# ================================================
# Entrypoint of the script
# ================================================
echo Starting $0 version ${SCRIPTVERSION} at ` date `...
SplitPath $0
FFPROOT=${SPLITTEDPATH}

if [ $# -eq 0 ]; then
    PrintHelp
    exit 0
fi

case $1 in
	StartFFP)
		ReadParams $@
		StartFFP
		;;
	StopFFP)
		ReadParams $@
		StopFFP
		;;
	InvokeHelper)
		ReadParams $@
		InvokeHelper
		;;
	Test)
		ReadParams $@
		tmpfile=/tmp/` date +%s%N `
		WriteParams ${tmpfile}
		cat ${tmpfile}
		rm ${tmpfile}
		;;
	*)
		if [ $# -eq 3 ]; then	
			# Assume this is an 'old style' call
			FFPROOT=$1
			NASTYPE=$2
			FIRMWARE=$3
			StartFFP
		else
			PrintHelp
		fi
		;;
esac

